<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Followupemail2
 * @version    2.2.3
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Followupemail2\Controller\Cart;

use Aheadworks\Followupemail2\Model\Event\Queue\CartRestorer;
use Magento\Framework\App\Action\Context;

/**
 * Class Restore
 * @package Aheadworks\Followupemail2\Controller\Cart
 */
class Restore extends \Magento\Framework\App\Action\Action
{
    /**
     * @var CartRestorer
     */
    private $cartRestorer;

    /**
     * @param Context $context
     * @param CartRestorer $cartRestorer
     */
    public function __construct(
        Context $context,
        CartRestorer $cartRestorer
    ) {
        parent::__construct($context);
        $this->cartRestorer = $cartRestorer;
    }

    /**
     * {@inheritdoc}
     */
    public function execute()
    {
        /** @var \Magento\Framework\Controller\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();
        $code = $this->getRequest()->getParam('code');
        if ($code) {
            try {
                if ($this->cartRestorer->restore($code)) {
                    #############################################################################################
                    # fix necessary to allow the passage to the cart page of analytics tracking.
                    # The setPath() method previously used by the module allowed (by adding a second parameter)
                    # the passing of parameters to the url but incorrectly for analytics so it was replaced
                    # by the setUrl() method
                    $requestParams = $this->getRequest()->getParams();
                    unset($requestParams['code']);
                    $params = http_build_query($requestParams);
                    $url = $this->_url->getUrl('checkout/cart')."?".$params;
                    return $resultRedirect->setUrl($url);
                    #############################################################################################

                    //return $resultRedirect->setPath('checkout/cart');
                }
            } catch (\Exception $e) {
                $this->messageManager->addErrorMessage($e->getMessage());
                return $resultRedirect->setPath('checkout/cart');
            }
        }
        $this->messageManager->addErrorMessage(__('Wrong code specified'));
        return $resultRedirect->setPath('/');
    }
}
